@@ -1,4 +1,4 @@
-__version__ = "0.6.13"
+__version__ = "0.6.15"
 
 from .core import (
     magicclass,
@@ -7,6 +7,7 @@ from .core import (
     magictoolbar,
     Parameters,
     build_help,
+    get_button,
     get_function_gui,
     repeat,
     update_widget_state,
@@ -21,6 +22,7 @@ from .wrappers import (
     nogui,
     mark_preview,
     impl_preview,
+    setup_function_gui,
     mark_on_calling,
     mark_on_called,
     abstractapi,
@@ -49,6 +51,7 @@ __all__ = [
     "magictoolbar",
     "Parameters",
     "build_help",
+    "get_button",
     "get_function_gui",
     "repeat",
     "update_widget_state",
@@ -60,6 +63,7 @@ __all__ = [
     "nogui",
     "mark_preview",
     "impl_preview",
+    "setup_function_gui",
     "mark_on_calling",
     "mark_on_called",
     "abstractapi",
