@@ -15,7 +15,6 @@ from abc import ABCMeta
 from typing_extensions import _AnnotatedAlias, Literal
 import inspect
 import warnings
-from enum import Enum
 from qtpy.QtWidgets import QWidget, QDockWidget
 
 from psygnal import Signal
@@ -34,7 +33,6 @@ from magicgui.widgets import (
 from magicgui.application import use_app
 from magicclass._magicgui_compat import (
     ButtonWidget,
-    WidgetProtocol,
     Undefined,
     MGUI_SIMPLE_TYPES,
     ValueWidget,
@@ -50,10 +48,11 @@ from .mgui_ext import (
     _LabeledWidgetAction,
     mguiLike,
 )
-from .utils import copy_class, callable_to_classes
-from ._macro import GuiMacro
-from ._macro_utils import inject_recorder, value_widget_callback
+from .utils import copy_class, callable_to_classes, show_dialog_from_mgui
+from ._macro import GuiMacro, DummyMacro
+from ._macro_utils import inject_recorder, inject_silencer, value_widget_callback
 from ._icon import get_icon
+from ._gui_modes import PopUpMode, ErrorMode
 
 from magicclass.utils import (
     get_signature,
@@ -78,82 +77,11 @@ from magicclass.functools import wraps
 if TYPE_CHECKING:
     import numpy as np
     import napari
-    from types import TracebackType
     from typing_extensions import Self
 
     _X = TypeVar("_X", bound=MGUI_SIMPLE_TYPES)
     _M = TypeVar("_M", bound="MagicTemplate")
 
-
-class PopUpMode(Enum):
-    """Define how to popup FunctionGui."""
-
-    popup = "popup"
-    first = "first"
-    last = "last"
-    above = "above"
-    below = "below"
-    dock = "dock"
-    dialog = "dialog"
-    parentlast = "parentlast"
-    parentsub = "parentsub"
-
-
-def _msgbox_raising(e: Exception, parent: Widget):
-    from ._message_box import QtErrorMessageBox
-
-    return QtErrorMessageBox.raise_(e, parent=parent.native)
-
-
-def _stderr_raising(e: Exception, parent: Widget):
-    pass
-
-
-def _stdout_raising(e: Exception, parent: Widget):
-    print(f"{e.__class__.__name__}: {e}")
-
-
-def _napari_notification_raising(e: Exception, parent: Widget):
-    from napari.utils.notifications import show_error
-
-    return show_error(str(e))
-
-
-class ErrorMode(Enum):
-    msgbox = "msgbox"
-    stderr = "stderr"
-    stdout = "stdout"
-    napari = "napari"
-
-    def get_handler(self):
-        """Get error handler."""
-        return ErrorModeHandlers[self]
-
-    def wrap_handler(self, func: Callable, parent):
-        """Wrap function with the error handler."""
-        handler = self.get_handler()
-
-        @functools.wraps(func)
-        def wrapped_func(*args, **kwargs):
-            try:
-                out = func(*args, **kwargs)
-            except Exception as e:
-                e.__traceback__ = _cleanup_tb(e.__traceback__)
-                handler(e, parent=parent)
-                out = e
-            return out
-
-        return wrapped_func
-
-
-ErrorModeHandlers = {
-    ErrorMode.msgbox: _msgbox_raising,
-    ErrorMode.stderr: _stderr_raising,
-    ErrorMode.stdout: _stdout_raising,
-    ErrorMode.napari: _napari_notification_raising,
-}
-
-
 defaults = {
     "popup_mode": PopUpMode.popup,
     "error_mode": ErrorMode.msgbox,
@@ -171,7 +99,6 @@ _RESERVED = frozenset(
         "_popup_mode",
         "_my_symbol",
         "_macro_instance",
-        "macro",
         "annotation",
         "enabled",
         "find_ancestor",
@@ -254,7 +181,7 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
     __magicclass_parent__: None | MagicTemplate
     __magicclass_children__: list[MagicTemplate]
     _close_on_run: bool
-    _component_class: type[Action | Widget]
+    _component_class: type[Action | PushButtonPlus]
     _error_mode: ErrorMode
     _list: list[Action | Widget]
     _macro_instance: GuiMacro
@@ -279,7 +206,7 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
     native: QWidget
     options: dict
     param_kind: inspect._ParameterKind
-    parent: Widget
+    parent: Widget | None
     parent_changed: Signal
     tooltip: str
     visible: bool
@@ -310,6 +237,9 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
 
     if TYPE_CHECKING:
 
+        def __iter__(self) -> Iterator[_W]:
+            raise NotImplementedError()
+
         def index(self, value: Any, start: int, stop: int) -> int:
             raise NotImplementedError()
 
@@ -344,14 +274,9 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
     def parent_viewer(self) -> napari.Viewer | None:
         """Return napari.Viewer if magic class is a dock widget of a viewer."""
         parent_self = self._search_parent_magicclass()
-        if parent_self.native.parent() is None:
+        if not isinstance(parent_self.native.parent(), QDockWidget):
             return None
-        try:
-            from napari.utils._magicgui import find_viewer_ancestor
-        except ImportError:
-            return None
-        viewer = find_viewer_ancestor(parent_self.native)
-        return viewer
+        return _find_viewer_ancestor(parent_self.native)
 
     @property
     def parent_dock_widget(self) -> QDockWidget | None:
@@ -588,6 +513,7 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
                     child_widget.changed.connect(widget.changed)
                     child_widget.tooltip = widget.tooltip
                     child_widget._doc = widget._doc
+                    child_widget._get_running = lambda: widget.running
 
                 widget._unwrapped = True
 
@@ -601,33 +527,33 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
             )
 
     # fmt: off
+    @overload
+    @classmethod
+    def field(cls, gui_class: type[_M], *, name: str | None = None, label: str | None = None, widget_type: str | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[_M, Any]: ...  # noqa
+
     @overload
     @classmethod
     def field(cls, type_of_widget: type[_W], *, name: str | None = None, label: str | None = None, options: dict[str, Any] = {}, record: bool = True) -> MagicField[_W, Any]: ...  # noqa
 
     @overload
     @classmethod
-    def field(cls, obj: type[_X], *, name: str | None = None, label: str | None = None, widget_type: str | type[WidgetProtocol] | type[Widget] | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[ValueWidget, _X]: ...  # noqa
+    def field(cls, obj: type[_X], *, name: str | None = None, label: str | None = None, widget_type: str | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[ValueWidget, _X]: ...  # noqa
 
     @overload
     @classmethod
-    def field(cls, obj: _X, *, name: str | None = None, label: str | None = None, widget_type: str | type[WidgetProtocol] | type[Widget] | None = None, options: dict[str, Any] = {}, record: bool = True) -> MagicField[ValueWidget, _X]: ...  # noqa
+    def field(cls, obj: _X, *, name: str | None = None, label: str | None = None, widget_type: str | None = None, options: dict[str, Any] = {}, record: bool = True) -> MagicField[ValueWidget, _X]: ...  # noqa
 
     @overload
     @classmethod
-    def field(cls, gui_class: type[_M], *, name: str | None = None, label: str | None = None, widget_type: str | type[WidgetProtocol] | type[Widget] | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[_M, Any]: ...  # noqa
+    def field(cls, obj: Any | None, *, name: str | None = None, label: str | None = None, widget_type: type[_W] = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[_W, Any]: ...  # noqa
 
     @overload
     @classmethod
-    def field(cls, obj: Any, *, name: str | None = None, label: str | None = None, widget_type: type[_W] = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[_W, Any]: ...  # noqa
+    def field(cls, obj: Any, *, name: str | None = None, label: str | None = None, widget_type: str | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[Widget, Any]: ...  # noqa
 
     @overload
     @classmethod
-    def field(cls, obj: Any, *, name: str | None = None, label: str | None = None, widget_type: str | type[WidgetProtocol] | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[Widget, Any]: ...  # noqa
-
-    @overload
-    @classmethod
-    def field(cls, *, name: str | None = None, label: str | None = None, widget_type: str | type[WidgetProtocol] | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[Widget, Any]: ...  # noqa
+    def field(cls, *, name: str | None = None, label: str | None = None, widget_type: str | type[Widget] | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicField[Widget, Any]: ...  # noqa
     # fmt: on
 
     @classmethod
@@ -692,11 +618,11 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
 
     @overload
     @classmethod
-    def vfield(cls, obj: _X, *, name: str | None = None, label: str | None = None, widget_type: str | type[WidgetProtocol] | type[Widget] | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicValueField[ValueWidget, _X]: ...  # noqa
+    def vfield(cls, annotation: type[_X], *, name: str | None = None, label: str | None = None, widget_type: str | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicValueField[ValueWidget, _X]: ...  # noqa
 
     @overload
     @classmethod
-    def vfield(cls, annotation: type[_X], *, name: str | None = None, label: str | None = None, widget_type: str | type[WidgetProtocol] | type[Widget] | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicValueField[ValueWidget, _X]: ...  # noqa
+    def vfield(cls, obj: _X, *, name: str | None = None, label: str | None = None, widget_type: str | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicValueField[ValueWidget, _X]: ...  # noqa
 
     @overload
     @classmethod
@@ -704,7 +630,7 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
 
     @overload
     @classmethod
-    def vfield(cls, obj: Any, *, name: str | None = None, label: str | None = None, widget_type: str | type[WidgetProtocol] | type[Widget] | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicValueField[Widget, Any]: ...  # noqa
+    def vfield(cls, obj: Any, *, name: str | None = None, label: str | None = None, widget_type: str | type[Widget] | None = None, options: dict[str, Any] = {}, record: bool = True, ) -> MagicValueField[Widget, Any]: ...  # noqa
     # fmt: on
 
     @classmethod
@@ -793,20 +719,12 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
 
         elif nparams == 1 and issubclass(fgui_classes[0], FileEdit) and not has_preview:
             # We don't want to open a magicgui dialog and again open a file dialog.
+
             def run_function():
                 mgui = _build_mgui(widget, func, self)
                 mgui.native.setParent(self.native, mgui.native.windowFlags())
-                fdialog: FileEdit = mgui[0]
-                if result := fdialog._show_file_dialog(
-                    fdialog.mode,
-                    caption=fdialog._btn_text,
-                    start_path=str(fdialog.value),
-                    filter=fdialog.filter,
-                ):
-                    fdialog.value = result
-                    out = mgui(result)
-                else:
-                    out = None
+                out = show_dialog_from_mgui(mgui)
+                self._popup_mode.connect_close_callback(mgui)
                 return out
 
         else:
@@ -814,12 +732,7 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
 
             def run_function():
                 mgui = _build_mgui(widget, func, self)
-                _need_title_bar = self._popup_mode not in (
-                    PopUpMode.popup,
-                    PopUpMode.dock,
-                    PopUpMode.parentsub,
-                    PopUpMode.dialog,
-                )
+                _need_title_bar = self._popup_mode.need_title_bar()
                 if mgui.call_count == 0:  # connect only once
                     _prep_func(mgui)
                     if _need_title_bar:
@@ -836,16 +749,7 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
                             mgui.insert(0, title)
 
                     if self._close_on_run and not mgui._auto_call:
-                        if self._popup_mode not in (
-                            PopUpMode.dock,
-                            PopUpMode.parentsub,
-                            PopUpMode.dialog,
-                        ):
-                            mgui.called.connect(mgui.hide)
-                        elif self._popup_mode in (PopUpMode.dock, PopUpMode.parentsub):
-                            # If FunctioGui is docked or in a subwindow, we should close
-                            # the parent QDockWidget/QMdiSubwindow.
-                            mgui.called.connect(lambda: mgui.parent.hide())
+                        self._popup_mode.connect_close_callback(mgui)
 
                 if nparams == 1 and issubclass(fgui_classes[0], FileEdit):
                     fdialog: FileEdit = mgui[int(_need_title_bar)]
@@ -859,19 +763,7 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
                     else:
                         return None
 
-                if self._popup_mode not in (
-                    PopUpMode.dock,
-                    PopUpMode.dialog,
-                    PopUpMode.parentsub,
-                ):
-                    widget.mgui.show()
-                elif self._popup_mode == PopUpMode.dock:
-                    mgui.parent.show()  # show dock widget
-                elif self._popup_mode == PopUpMode.parentsub:
-                    mgui.native.parent().setVisible(True)
-                else:
-                    mgui.exec_as_dialog(parent=self)
-
+                self._popup_mode.activate_magicgui(mgui)
                 return None
 
         widget.changed.connect(run_function)
@@ -903,7 +795,7 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
             yield from child._iter_child_magicclasses()
 
     def _call_with_return_callback(self, fname: str, *args, **kwargs) -> None:
-        from ..core import get_function_gui
+        from magicclass.core import get_function_gui
 
         fgui = get_function_gui(self, fname)
         fgui(*args, **kwargs)
@@ -911,10 +803,13 @@ class MagicTemplate(MutableSequence[_W], metaclass=_MagicTemplateMeta):
 
 
 class BaseGui(MagicTemplate[_W]):
-    def __init__(self, close_on_run, popup_mode, error_mode):
+    def __init__(
+        self, close_on_run=True, popup_mode=PopUpMode.popup, error_mode=ErrorMode.msgbox
+    ):
         self._macro_instance = GuiMacro(
             max_lines=defaults["macro-max-history"],
             flags={"Get": False, "Return": False},
+            ui=self,
         )
         self.__magicclass_parent__: BaseGui | None = None
         self.__magicclass_children__: list[MagicTemplate] = []
@@ -924,10 +819,9 @@ class BaseGui(MagicTemplate[_W]):
         self._my_symbol = Symbol.var("ui")
         self._icon = None
 
-        self.macro.widget.__magicclass_parent__ = self
-
     @property
     def icon(self):
+        """Icon of this GUI."""
         return self._icon
 
     @icon.setter
@@ -1090,7 +984,13 @@ def _create_gui_method(self: BaseGui, obj: MethodType):
     for param in func.__signature__.parameters.values():
         if isinstance(param.annotation, _AnnotatedAlias):
             annot, opt = split_annotated_type(param.annotation)
-            param = MagicParameter.from_parameter(param, gui_options=opt)
+            param = MagicParameter(
+                param.name,
+                param.kind,
+                default=param.default,
+                annotation=annot,
+                gui_options=opt,
+            )
 
         if isinstance(param, MagicParameter):
             _param = MagicParameter(
@@ -1147,6 +1047,7 @@ def _create_gui_method(self: BaseGui, obj: MethodType):
 
 
 def _build_mgui(widget_: Action | PushButtonPlus, func: Callable, parent: BaseGui):
+    """Build a magicgui from a function for the give button/action."""
     if widget_.mgui is not None:
         return widget_.mgui
     try:
@@ -1171,24 +1072,29 @@ def _build_mgui(widget_: Action | PushButtonPlus, func: Callable, parent: BaseGu
         mgui = FunctionGuiPlus(
             func, call_button, layout=layout, labels=labels, auto_call=auto_call
         )
+        # set function GUI.
+        widget_.mgui = mgui
+        name = widget_.name or ""
+        mgui.native.setWindowTitle(name.replace("_", " ").strip())
+
         preview_setting = opt.get("preview", None)
         if preview_setting is not None:
             btn_text, is_auto_call, previewer = preview_setting
             mgui.append_preview(
                 previewer.__get__(parent), btn_text, auto_call=is_auto_call
             )
+        setup_func = opt.get("setup", None)
+        if setup_func is not None:
+            setup_func(parent, mgui)
 
     except Exception as e:
         msg = (
             "Exception was raised during building magicgui from method "
             f"{func.__name__}.\n{e.__class__.__name__}: {e}"
         )
+        widget_.mgui = None
         raise type(e)(msg)
 
-    widget_.mgui = mgui
-    name = widget_.name or ""
-    mgui.native.setWindowTitle(name.replace("_", " ").strip())
-
     return _connect_functiongui_event(mgui, opt)
 
 
@@ -1236,7 +1142,9 @@ def _child_that_has_widget(
     return child_instance
 
 
-def convert_attributes(cls: type[_T], hide: tuple[type, ...]) -> dict[str, Any]:
+def convert_attributes(
+    cls: type[_T], hide: tuple[type, ...], record: bool = True
+) -> dict[str, Any]:
     """
     Convert class attributes into macro recordable ones.
 
@@ -1258,7 +1166,7 @@ def convert_attributes(cls: type[_T], hide: tuple[type, ...]) -> dict[str, Any]:
         New namespace.
     """
     _dict: dict[str, Callable] = {}
-    _pass_type = (
+    _pass = (
         property,
         classmethod,
         staticmethod,
@@ -1268,24 +1176,44 @@ def convert_attributes(cls: type[_T], hide: tuple[type, ...]) -> dict[str, Any]:
         abstractapi,
     )
     mro = [c for c in cls.__mro__ if c not in hide]
+    default = None if record else "false"
     for subcls in reversed(mro):
         for name, obj in subcls.__dict__.items():
+            _isfunc = callable(obj)
             if isinstance(obj, _MagicTemplateMeta):
                 new_attr = copy_class(obj, cls, name=name)
-            elif (
-                name.startswith("_") or isinstance(obj, _pass_type) or not callable(obj)
-            ):
+            elif name.startswith("_") or isinstance(obj, _pass) or not _isfunc:
                 # private method, non-action-like object, not-callable object are passed.
                 new_attr = obj
-            elif callable(obj) and get_additional_option(obj, "record", True):
-                new_attr = inject_recorder(obj)
+            elif _isfunc:
+                _record_policy = get_additional_option(obj, "record", default)
+                if _record_policy is None:
+                    new_attr = inject_recorder(obj)
+                elif _record_policy == "false":
+                    new_attr = obj
+                elif _record_policy == "all-false":
+                    new_attr = inject_silencer(obj)
+                else:
+                    raise ValueError(f"Invalid record policy: {_record_policy}")
             else:
                 new_attr = obj
 
             _dict[name] = new_attr
+    # replace the macro object with the dummy one.
+    if not record:
+        _dict["macro"] = macro
     return _dict
 
 
+_dummy_macro = DummyMacro()
+
+
+@property
+def macro(self: BaseGui):
+    """Return the dummy macro object"""
+    return _dummy_macro
+
+
 def _define_popup(self: BaseGui, obj, widget: PushButtonPlus | Action):
     # deal with popup mode.
     popup_mode = self._popup_mode
@@ -1293,8 +1221,10 @@ def _define_popup(self: BaseGui, obj, widget: PushButtonPlus | Action):
         # To be popped up correctly, window flags of FunctionGui should be
         # "windowFlags" and should appear at the center.
         def _prep(mgui: FunctionGui):
-            mgui.native.setParent(self.native, mgui.native.windowFlags())
-            move_to_screen_center(mgui.native)
+            if mgui.native.parent() is not self.native:
+                mgui.native.setParent(self.native, mgui.native.windowFlags())
+                mgui.show()
+                move_to_screen_center(mgui.native)
 
     elif popup_mode == PopUpMode.parentlast:
 
@@ -1417,19 +1347,19 @@ def _implement_confirmation(
     return _method
 
 
-def _cleanup_tb(tb: TracebackType):
-    """Remove useless info from a traceback object."""
-    current_tb = tb
-    while current_tb is not None:
-        if current_tb.tb_frame.f_code.co_name == "_recordable":
-            tb = current_tb.tb_next
-            break
-        current_tb = current_tb.tb_next
-    return tb
-
-
 def _empty_func(name: str) -> Callable[[Any], None]:
     """Create a named function that does nothing."""
     f = lambda x: None
     f.__name__ = name
     return f
+
+
+def _find_viewer_ancestor(widget: QWidget) -> napari.Viewer | None:
+    """Return the closest parent napari Viewer."""
+    parent = widget.parent()
+    while parent:
+        if hasattr(parent, "_qt_viewer"):  # QMainWindow
+            return parent._qt_viewer.viewer
+
+        parent = parent.parent()
+    return None
