@@ -21,9 +21,6 @@ from magicclass._magicgui_compat import (
 )
 from macrokit import Symbol
 
-from magicclass.wrappers import abstractapi
-
-
 from .keybinding import register_shortcut
 from .mgui_ext import PushButtonPlus
 from .toolbar import ToolBarGui, QtTabToolBar
@@ -267,7 +264,7 @@ class ClassGuiBase(BaseGui[Widget]):
 
                     # Now, "widget" is a Widget object. Add widget in a way similar to "insert" method
                     # of Container.
-                    if name.startswith("_"):
+                    if widget.name.startswith("_"):
                         continue
 
                     moveto = get_additional_option(attr, "into")
@@ -337,9 +334,6 @@ class ClassGuiBase(BaseGui[Widget]):
                     self.__magicclass_children__.append(widget)
                     widget._my_symbol = Symbol(widget.name)
 
-                # NOTE: This is not safe. Attributes could collision and macro recording
-                # may break if not correctly named.
-
         _widget = widget
 
         if self.labels:
@@ -400,7 +394,7 @@ def make_gui(
         )
 
         def __init__(
-            self: cls,
+            self: ClassGuiBase,
             layout: str = "vertical",
             close_on_run: bool = None,
             popup_mode: str | PopUpMode = None,
@@ -409,7 +403,6 @@ def make_gui(
             name: str = None,
             visible: bool = None,
         ):
-
             container.__init__(
                 self, layout=layout, labels=labels, name=name, visible=visible
             )
@@ -428,7 +421,7 @@ def make_gui(
 
         # ui["x"] will not return widget if x is a MagicValueField.
         # To ensure __getitem__ returns a Widget, this method should be overriden.
-        def __getitem__(self: cls, key):
+        def __getitem__(self: ClassGuiBase, key):
             """Get item by integer, str, or slice."""
             if isinstance(key, str):
                 for widget in self._list:
@@ -436,18 +429,18 @@ def make_gui(
                         return widget
             return container.__getattr__(self, key)
 
-        def __setattr__(self: cls, name: str, value: Any) -> None:
+        def __setattr__(self, name: str, value: Any) -> None:
             if not isinstance(getattr(self.__class__, name, None), MagicField):
                 container.__setattr__(self, name, value)
             else:
                 object.__setattr__(self, name, value)
 
-        def insert(self: cls, key: int, widget: Widget) -> None:
+        def insert(self: ClassGuiBase, key: int, widget: Widget) -> None:
             self._fast_insert(key, widget)
             self._unify_label_widths()
             return None
 
-        def reset_choices(self: cls, *_: Any):
+        def reset_choices(self: ClassGuiBase, *_: Any):
             """Reset child Categorical widgets"""
             all_widgets: set[Widget] = set()
 
@@ -462,7 +455,7 @@ def make_gui(
                     w.reset_choices()
             return None
 
-        def show(self: cls, run: bool = True) -> None:
+        def show(self: ClassGuiBase, run: bool = True) -> None:
             """
             Show GUI. If any of the parent GUI is a dock widget in napari, then this
             will also show up as a dock widget (floating if in popup mode).
@@ -474,11 +467,10 @@ def make_gui(
                 to close the window immediately. If true, application gets executed if
                 needed.
             """
-            if self.__magicclass_parent__ is not None and self.parent is None:
+            mcls_parent = self.__magicclass_parent__
+            if mcls_parent is not None and self.parent is None:
                 # If child magic class is closed before, we have to set parent again.
-                self.native.setParent(
-                    self.__magicclass_parent__.native, self.native.windowFlags()
-                )
+                self.native.setParent(mcls_parent.native, self.native.windowFlags())
 
             viewer = self.parent_viewer
             if viewer is not None and self.parent is not None:
@@ -499,12 +491,17 @@ def make_gui(
                     dock.setFloating(_floating)
             else:
                 container.show(self, run=False)
+                if mcls_parent is not None:
+                    topleft = mcls_parent.native.geometry().topLeft()
+                    topleft.setX(topleft.x() + 20)
+                    topleft.setY(topleft.y() + 20)
+                    self.native.move(topleft)
                 self.native.activateWindow()
                 if run:
                     run_app()
             return None
 
-        def close(self: cls):
+        def close(self: ClassGuiBase):
             """Close GUI. if this widget is a dock widget, then also close it."""
 
             current_self = self._search_parent_magicclass()
@@ -523,13 +520,13 @@ def make_gui(
         if issubclass(container, MainWindow):
             # Similar to napari's viewer.window.add_dock_widget.
             # See napari/_qt/widgets/qt_viewer_dock_widget.py
-            from ..wrappers import nogui
+            from magicclass.wrappers import nogui
 
             # This function will be detected as non-reserved method so that magicclass will
             # try to convert it into widget. Should decorate with @nogui.
             @nogui
             def add_dock_widget(
-                self: cls,
+                self: MainWindowClassGui,
                 widget: Widget,
                 *,
                 name: str = "",
@@ -570,7 +567,7 @@ def make_gui(
                     widget._my_symbol = Symbol(name)
 
             @nogui
-            def remove_dock_widget(self: cls, widget: Widget):
+            def remove_dock_widget(self: MainWindowClassGui, widget: Widget):
                 from ._dock_widget import QtDockWidget
 
                 dock = None
@@ -590,12 +587,12 @@ def make_gui(
                 self.__magicclass_children__.pop(i_dock)
 
             @property
-            def status(self: cls) -> str:
+            def status(self: MainWindowClassGui) -> str:
                 """Get status tip."""
                 return self.native.statusTip()
 
             @status.setter
-            def status(self: cls, text: str):
+            def status(self: MainWindowClassGui, text: str):
                 """Set status tip."""
                 self.native.setStatusTip(text)
                 self.native.statusBar().showMessage(text, 5000)
