@@ -7,8 +7,6 @@ from macrokit import Symbol
 from qtpy.QtWidgets import QMenu
 from qtpy.QtCore import Qt
 
-from magicclass.wrappers import abstractapi
-
 from .mgui_ext import AbstractAction, WidgetAction, _LabeledWidgetAction
 from .keybinding import register_shortcut
 from ._base import (
@@ -147,7 +145,7 @@ class MenuGuiBase(ContainerLikeGui):
                         # with a type object (not instance).
                         widget.__magicclass_parent__ = self
 
-                    if name.startswith("_"):
+                    if widget.name.startswith("_"):
                         continue
                     moveto = get_additional_option(attr, "into")
                     copyto = get_additional_option(attr, "copyto", [])
