@@ -1,5 +1,6 @@
 from __future__ import annotations
-from typing import Callable, Iterable, Any, Generic, TypeVar
+from typing import Callable, Iterable, Any, Generic, TypeVar, Union
+from typing_extensions import TypeGuard
 from qtpy.QtWidgets import (
     QPushButton,
     QAction,
@@ -16,6 +17,11 @@ from ._function_gui import FunctionGuiPlus
 from ._icon import get_icon
 
 # magicgui widgets that need to be extended to fit into magicclass
+Clickable = Union["PushButtonPlus", "Action"]
+
+
+def is_clickable(wdt: Widget) -> TypeGuard[Clickable]:
+    return isinstance(wdt, (PushButtonPlus, Action))
 
 
 class PushButtonPlus(PushButton):
@@ -28,11 +34,14 @@ class PushButtonPlus(PushButton):
         self.mgui: FunctionGuiPlus | None = None  # tagged function GUI
         self._doc = ""
         self._unwrapped = False
+        self._get_running: Callable[[], bool] | None = None
 
     @property
     def running(self) -> bool:
         """Return true if embedded magicgui widget is running from GUI."""
-        return getattr(self.mgui, "running", False)
+        if self._get_running is None:
+            return getattr(self.mgui, "running", False)
+        return self._get_running()
 
     def set_shortcut(self, key):
         """Set keyboard shortcut to the button."""
@@ -249,10 +258,14 @@ class Action(AbstractAction):
         self._callbacks = []
 
         self.native.triggered.connect(lambda: self.changed.emit(self.value))
+        self._get_running: Callable[[], bool] | None = None
 
     @property
     def running(self) -> bool:
-        return getattr(self.mgui, "running", False)
+        """Return true if embedded magicgui widget is running from GUI."""
+        if self._get_running is None:
+            return getattr(self.mgui, "running", False)
+        return self._get_running()
 
     def set_shortcut(self, key):
         self.native.setShortcut(key)
@@ -289,6 +302,9 @@ class Action(AbstractAction):
         icon.install(self)
         self._icon = icon
 
+    def trigger(self):
+        return self.native.trigger()
+
     def from_options(self, options: dict[str] | Callable):
         if callable(options):
             try:
