@@ -205,7 +205,7 @@ class ToolBarGui(ContainerLikeGui):
                         # with a type object (not instance).
                         widget.__magicclass_parent__ = self
 
-                    if name.startswith("_"):
+                    if widget.name.startswith("_"):
                         continue
 
                     moveto = get_additional_option(attr, "into")
