@@ -19,10 +19,11 @@ if not MAGICGUI_BEFORE_0_6:
 
 from macrokit import Symbol
 
-from ..signature import split_annotated_type
+from magicclass.signature import split_annotated_type
 
 if TYPE_CHECKING:
     from ._base import BaseGui
+    from magicgui.widgets import FunctionGui, FileEdit
 
 
 def get_parameters(fgui: FunctionGui):
@@ -113,6 +114,22 @@ def callable_to_classes(f: Callable) -> list[type[Widget]]:
     return [_parameter_to_widget_class(p) for p in sig.parameters.values()]
 
 
+def show_dialog_from_mgui(mgui: FunctionGui):
+    """Show file dialog from given magicgui widget."""
+    fdialog: FileEdit = mgui[0]
+    if result := fdialog._show_file_dialog(
+        fdialog.mode,
+        caption=fdialog._btn_text,
+        start_path=str(fdialog.value),
+        filter=fdialog.filter,
+    ):
+        fdialog.value = result
+        out = mgui(result)
+    else:
+        out = None
+    return out
+
+
 TZ_EMPTY = "__no__default__"
 
 
