@@ -1,8 +1,10 @@
 from __future__ import annotations
 from functools import wraps as functools_wraps
 import inspect
+from types import ModuleType
 from weakref import WeakValueDictionary
 from typing import Any, TYPE_CHECKING, Callable
+from macrokit import Symbol, Expr
 
 from magicclass._gui.class_gui import (
     ClassGuiBase,
@@ -33,19 +35,24 @@ from magicclass._gui._base import (
 from magicclass._gui import ContextMenuGui, MenuGui, ToolBarGui
 from magicclass._app import get_app
 from magicclass.types import WidgetType
-from magicclass import _register_types  # activate type registration things.
+
+# activate type registration things.
+from magicclass import _register_types
+
+del _register_types
 
 if TYPE_CHECKING:
     from magicclass.stylesheets import StyleSheet
     from magicclass._gui import MenuGuiBase
+    from magicclass._gui.mgui_ext import Clickable
     from magicclass._gui._function_gui import FunctionGuiPlus
     from magicclass.types import WidgetTypeStr, PopUpModeStr, ErrorModeStr
     from magicclass.help import HelpWidget
     from macrokit import Macro
 
-_BASE_CLASS_SUFFIX = "_Base"
+_BASE_CLASS_SUFFIX = ".base"
 
-_TYPE_MAP = {
+_TYPE_MAP: dict[WidgetType, type[ClassGuiBase]] = {
     WidgetType.none: ClassGui,
     WidgetType.scrollable: ScrollableClassGui,
     WidgetType.draggable: DraggableClassGui,
@@ -78,6 +85,8 @@ def magicclass(
     icon: Any | None = None,
     stylesheet: str | StyleSheet = None,
     properties: dict[str, Any] = None,
+    record: bool = True,
+    symbol: str = "ui",
 ):
     """
     Decorator that can convert a Python class into a widget.
@@ -103,8 +112,6 @@ def magicclass(
     close_on_run : bool, default is True
         If True, magicgui created by every method will be deleted after the method is
         completed without exceptions, i.e. magicgui is more like a dialog.
-    popup : bool, default is True
-        Deprecated.
     popup_mode : str or PopUpMode, default is PopUpMode.popup
         Option of how to popup FunctionGui widget when a button is clicked.
     error_mode : str or ErrorMode, default is ErrorMode.msgbox
@@ -118,6 +125,10 @@ def magicclass(
     properties : dict, optional
         Set properties to the widget if given. This argument is useful when you want
         to set width, height or margin without defining __post_init__.
+    record : bool, default is True
+        If True, macro recording is enabled.
+    symbol : str, default is "ui"
+        The identifier used in macro to represent this widget.
 
     Returns
     -------
@@ -151,7 +162,7 @@ def magicclass(
         mod = cls.__module__
         qualname = cls.__qualname__
 
-        new_attrs = convert_attributes(cls, hide=class_gui.__mro__)
+        new_attrs = convert_attributes(cls, hide=class_gui.__mro__, record=record)
         oldclass = type(cls.__name__ + _BASE_CLASS_SUFFIX, (cls,), {})
         newclass = type(cls.__name__, (class_gui, oldclass), new_attrs)
 
@@ -184,13 +195,7 @@ def magicclass(
                 gui_kwargs.update(kwargs)
                 kwargs = {}
 
-            class_gui.__init__(
-                self,
-                **gui_kwargs,
-            )
-            # prepare macro
-            macrowidget = self.macro.widget.native
-            macrowidget.setParent(self.native, macrowidget.windowFlags())
+            class_gui.__init__(self, **gui_kwargs)
 
             with self.macro.blocked():
                 super(oldclass, self).__init__(*args, **kwargs)
@@ -210,6 +215,7 @@ def magicclass(
             if properties:
                 for k, v in properties.items():
                     setattr(self, k, v)
+            self._my_symbol = _as_symbol(symbol)
 
         newclass.__init__ = __init__
 
@@ -233,6 +239,7 @@ def magicmenu(
     labels: bool = True,
     name: str | None = None,
     icon: Any | None = None,
+    record: bool = True,
 ):
     """Decorator that converts a Python class into a menu bar."""
     return _call_magicmenu(**locals(), menugui_class=MenuGui)
@@ -248,6 +255,7 @@ def magiccontext(
     labels: bool = True,
     name: str | None = None,
     icon: Any | None = None,
+    record: bool = True,
 ):
     """Decorator that converts a Python class into a context menu."""
 
@@ -279,7 +287,7 @@ def magiccontext(
         mod = cls.__module__
         qualname = cls.__qualname__
 
-        new_attrs = convert_attributes(cls, hide=ContextMenuGui.__mro__)
+        new_attrs = convert_attributes(cls, hide=ContextMenuGui.__mro__, record=record)
         oldclass = type(cls.__name__ + _BASE_CLASS_SUFFIX, (cls,), {})
         newclass = type(cls.__name__, (ContextMenuGui, oldclass), new_attrs)
 
@@ -306,13 +314,7 @@ def magiccontext(
                 gui_kwargs.update(kwargs)
                 kwargs = {}
 
-            ContextMenuGui.__init__(
-                self,
-                **gui_kwargs,
-            )
-
-            macrowidget = self.macro.widget.native
-            macrowidget.setParent(self.native, macrowidget.windowFlags())
+            ContextMenuGui.__init__(self, **gui_kwargs)
 
             with self.macro.blocked():
                 super(oldclass, self).__init__(*args, **kwargs)
@@ -348,6 +350,7 @@ def magictoolbar(
     labels: bool = True,
     name: str | None = None,
     icon: Any | None = None,
+    record: bool = True,
 ):
     """Decorator that converts a Python class into a menu bar."""
     return _call_magicmenu(**locals(), menugui_class=ToolBarGui)
@@ -361,6 +364,7 @@ def _call_magicmenu(
     labels: bool = True,
     name: str = None,
     icon: Any | None = None,
+    record: bool = True,
     menugui_class: type[MenuGuiBase] = None,
 ):
     """
@@ -408,7 +412,7 @@ def _call_magicmenu(
         mod = cls.__module__
         qualname = cls.__qualname__
 
-        new_attrs = convert_attributes(cls, hide=menugui_class.__mro__)
+        new_attrs = convert_attributes(cls, hide=menugui_class.__mro__, record=record)
         oldclass = type(cls.__name__ + _BASE_CLASS_SUFFIX, (cls,), {})
         newclass = type(cls.__name__, (menugui_class, oldclass), new_attrs)
 
@@ -435,13 +439,7 @@ def _call_magicmenu(
                 gui_kwargs.update(kwargs)
                 kwargs = {}
 
-            menugui_class.__init__(
-                self,
-                **gui_kwargs,
-            )
-
-            macrowidget = self.macro.widget.native
-            macrowidget.setParent(self.native, macrowidget.windowFlags())
+            menugui_class.__init__(self, **gui_kwargs)
 
             with self.macro.blocked():
                 super(oldclass, self).__init__(*args, **kwargs)
@@ -497,25 +495,73 @@ def build_help(ui: MagicTemplate, parent=None) -> HelpWidget:
     return help_widget
 
 
-def get_function_gui(ui: MagicTemplate, name: str) -> FunctionGuiPlus:
+def get_button(ui: MagicTemplate | ModuleType, name: str | None = None) -> Clickable:
     """
-    Get the FunctionGui object hidden beneath push button or menu.
+    Get the button/action object for the given method.
 
-    This function is a helper function for magicclass.
+    This function is a helper function for magicclass. Using this method is
+    always safer than directly accessing it by ``ui["method"]``.
+    Either of following expression is allowed.
 
-    Parameters
-    ----------
-    ui : MagicTemplate
-        Any of a magic-class instance.
-    name : str
-        Name of method (or strictly speaking, the name of PushButton).
+    >>> get_button(ui, "method")
+    >>> get_button(ui.method)
 
-    Returns
-    -------
-    FunctionGuiPlus
-        FunctionGui object.
     """
-    func = getattr(ui, name)
+    if name is None:
+        if hasattr(ui, "__self__") and callable(ui):
+            func = ui
+            ui = func.__self__
+            name = func.__name__
+        else:
+            raise TypeError(
+                "The first argument of `get_button() must be a method if "
+                "the method name is not given."
+            )
+    widget: Clickable = ui[name]
+
+    if not hasattr(widget, "mgui"):
+        raise TypeError(f"Widget {widget} does not have FunctionGui inside it.")
+
+    if widget._unwrapped:
+        from magicclass.signature import get_additional_option
+
+        opt = get_additional_option(getattr(ui, name), "into", None)
+        if opt is not None:
+            for child_instance in ui._iter_child_magicclasses():
+                _name = child_instance.__class__.__name__
+                if _name == opt:
+                    widget = child_instance[name]
+                    return widget
+            raise ValueError(f"Could not find {opt} in {ui}.")
+    return widget
+
+
+def get_function_gui(
+    ui: MagicTemplate | ModuleType, name: str = None
+) -> FunctionGuiPlus:
+    """
+    Get the FunctionGui object hidden beneath push button or menu action.
+
+    This function is a helper function for magicclass. Using this method is
+    always safer than directly accessing it by ``ui["method"].mgui``.
+    Either of following expression is allowed.
+
+    >>> get_function_gui(ui, "method")
+    >>> get_function_gui(ui.method)
+
+    """
+    if name is None:
+        if hasattr(ui, "__self__") and callable(ui):
+            func = ui
+            ui = func.__self__
+            name = func.__name__
+        else:
+            raise TypeError(
+                "The first argument of `get_function_gui() must be a method if "
+                "the method name is not given."
+            )
+    else:
+        func = getattr(ui, name)
     widget = ui[name]
 
     if not hasattr(widget, "mgui"):
@@ -568,7 +614,7 @@ def update_widget_state(ui: MagicTemplate, macro: Macro | str | None = None) ->
     macro : Macro or str, optional
         An executable macro or string that dictates how GUI will be updated.
     """
-    from macrokit import Head, Expr, Macro
+    from macrokit import Head, Macro
 
     if macro is None:
         macro = ui.macro
@@ -583,25 +629,23 @@ def update_widget_state(ui: MagicTemplate, macro: Macro | str | None = None) ->
         )
 
     for expr in macro:
-        if expr.head == Head.call:
+        if expr.head is Head.call:
             # ui.func(...)
-            ui_f, *arguments = expr.args
-            fname = str(ui_f.args[1])
+            fname = str(expr.at(0, 1))
+            args, kwargs = expr.eval_call_args()
             if fname.startswith("_"):
                 if fname != "_call_with_return_callback":
                     continue
-                args, kwargs = _arguments_to_values(arguments)
                 fgui = get_function_gui(ui, args[0])
 
             else:
-                args, kwargs = _arguments_to_values(arguments)
                 fgui = get_function_gui(ui, fname)
 
             with fgui.changed.blocked():
                 for key, value in kwargs.items():
                     getattr(fgui, key).value = value
 
-        elif expr.head == Head.assign:
+        elif expr.head is Head.assign:
             # ui.field.value = ...
             # ui.vfield = ...
             expr.eval({}, {str(ui._my_symbol): ui})
@@ -609,23 +653,6 @@ def update_widget_state(ui: MagicTemplate, macro: Macro | str | None = None) ->
     return None
 
 
-def _tuple(*args) -> tuple:
-    return args
-
-
-def _arguments_to_values(arguments) -> tuple[tuple, dict[str, Any]]:
-    from macrokit import Head, Expr, symbol
-
-    for i, arg in enumerate(arguments):
-        if isinstance(arg, Expr) and arg.head == Head.kw:
-            break
-    args = arguments[:i]
-    kwargs: list[Expr] = arguments[i:]
-    args = Expr(Head.call, [_tuple] + args).eval({symbol(_tuple): _tuple})
-    kwargs = Expr(Head.call, [dict] + kwargs).eval()
-    return args, kwargs
-
-
 class Parameters:
     def __init__(self):
         self.__name__ = self.__class__.__name__
@@ -675,3 +702,11 @@ class Parameters:
         """
         params = list(self.__signature__.parameters.keys())[1:]
         return {param: getattr(self, param) for param in params}
+
+
+def _as_symbol(s: str) -> Symbol | Expr:
+    if isinstance(s, str):
+        return Symbol(s)
+    elif not isinstance(s, (Symbol, Expr)):
+        raise TypeError(f"Expected a string or Symbol, got {type(s)}.")
+    return s
