@@ -1,5 +1,6 @@
 from __future__ import annotations
 from functools import wraps
+import inspect
 from typing import Any, Callable, TYPE_CHECKING
 from dask.diagnostics import Callback as DaskCallback
 from psygnal import Signal
@@ -14,13 +15,14 @@ from magicclass.utils.qthreading import (
 )
 
 if TYPE_CHECKING:
-    from ..._gui import BaseGui
+    from magicclass._gui import BaseGui
 
 
 class DaskProgressBar(DefaultProgressBar, DaskCallback):
     """A progress bar widget for dask computation."""
 
     computed = Signal(object)
+    new_cycle = Signal(int)
 
     def __init__(
         self,
@@ -33,7 +35,6 @@ class DaskProgressBar(DefaultProgressBar, DaskCallback):
         self._frac = 0.0
         self._n_computation = 0
         super().__init__(max=max)
-        self.footer[1].visible = self.footer[2].visible = False
         self._computed_signal = QtSignal()
         self._computed_signal.connect(self._on_computed)
 
@@ -45,7 +46,9 @@ class DaskProgressBar(DefaultProgressBar, DaskCallback):
     def _start(self, dsk):
         self._state = None
         self._frac = 0.0
+        self.new_cycle.emit(self._n_computation)
         self._n_computation += 1
+        self._timer.reset()
         self._start_thread()
         return None
 
@@ -71,34 +74,42 @@ class DaskProgressBar(DefaultProgressBar, DaskCallback):
         self._time_signal.emit()
         return None
 
-    def _finish(self, dsk, state, errored):
+    def _finish(self, dsk=None, state=None, errored=None):
         self._frac = 1.0
         self._running = False
         self._thread_timer.join()
-        self._timer.reset()
         return None
 
     def _on_timer_updated(self, _=None):
-        if self._n_computation > 1:
-            _prefix = f"({self._n_computation}) "
-        else:
-            _prefix = ""
         if self._timer.sec < 3600:
-            self.time_label.value = _prefix + self._timer.format_time(
-                "{min:0>2}:{sec:0>2}"
-            )
+            self.time_label.value = self._timer.format_time("{min:0>2}:{sec:0>2}")
         else:
-            self.time_label.value = _prefix + self._timer.format_time()
+            self.time_label.value = self._timer.format_time()
         return None
 
     def set_worker(self, worker: GeneratorWorker | FunctionWorker):
         """Set currently running worker."""
         self._worker = worker
-        if isinstance(self._worker, GeneratorWorker):
-            raise TypeError("Cannot set generator.")
-        self.footer[1].visible = False
-        self.footer[2].visible = False
-        self._time_signal.emit()
+        if not isinstance(self._worker, GeneratorWorker):
+            # FunctionWorker does not have yielded/aborted signals.
+            self.hide_footer()
+            return None
+        # initialize abort_button
+        self.abort_button.text = "Abort"
+        self.abort_button.changed.connect(self._abort_worker)
+        self.abort_button.enabled = True
+
+        # initialize pause_button
+        self.pause_button.text = "Pause"
+        self.pause_button.enabled = True
+        self.pause_button.changed.connect(self._toggle_pause)
+
+        @self._worker.paused.connect
+        def _on_pause():
+            self.pause_button.text = "Resume"
+            self.pause_button.enabled = True
+            self._timer.stop()
+
         return None
 
 
@@ -155,6 +166,7 @@ class dask_thread_worker(thread_worker):
                 "pbar": None,
                 "desc": "Progress",
                 "total": 100,
+                "descs": [],
             }
         else:
             self._progress["pbar"] = None
@@ -170,24 +182,73 @@ class dask_thread_worker(thread_worker):
         self._progressbars[gui_id] = pbar
         for c in self.computed._iter_as_method(gui):
             pbar.computed.connect(c)
+        if descs := self._progress.get("descs"):
+            if callable(descs):
+                arguments = self.__signature__.bind(gui, *args, **kwargs)
+                arguments.apply_defaults()
+                _descs = lambda: descs(**_filter_args(descs, arguments.arguments))
+            else:
+                _descs = lambda: iter(descs)
+            self._progress["desc"] = next(_descs(), "<No description>")
+            it = _descs()
+
+            @pbar.new_cycle.connect
+            def _(i: int):
+                pbar.set_description(next(it, "<No description>"))
+
         pbar.native.setParent(gui.native, self.__class__._WINDOW_FLAG)
         move_to_screen_center(pbar.native)
 
         worker = create_worker(
-            self._define_function(pbar).__get__(gui),
+            self._define_function(pbar, gui).__get__(gui),
             _ignore_errors=self._ignore_errors,
             _start_thread=False,
             *args,
             **kwargs,
         )
+        if self.is_generator:
+
+            @self.yielded.connect
+            def _(*args):
+                pbar.value = 0
+
+        @worker.returned.connect
+        def _on_returned(out):
+            if gui.macro.active and self._recorder is not None:
+                self._recorder(gui, out, *args, **kwargs)
+            return
 
         return worker
 
-    def _define_function(self, pbar):
-        @wraps(self._func)
-        def _wrapped(*args, **kwargs):
-            with pbar:
-                out = self._func(*args, **kwargs)
-            return out
+    def _define_function(self, pbar, gui: BaseGui):
+        if inspect.isgeneratorfunction(self._func):
+
+            @wraps(self._func)
+            def _wrapped(*args, **kwargs):
+                with pbar:
+                    with gui.macro.blocked():
+                        out = yield from self._func(*args, **kwargs)
+                return out
+
+        else:
+
+            @wraps(self._func)
+            def _wrapped(*args, **kwargs):
+                with pbar:
+                    with gui.macro.blocked():
+                        out = self._func(*args, **kwargs)
+                return out
 
         return _wrapped
+
+
+def _filter_args(fn: Callable, arguments: dict[str, Any]) -> dict[str, Any]:
+    sig = inspect.signature(fn)
+    params = sig.parameters
+    nparams = len(params)
+    if nparams == 0:
+        return {}
+    if list(sig.parameters.values())[-1] == inspect.Parameter.VAR_KEYWORD:
+        return arguments
+    existing_args = set(sig.parameters.keys())
+    return {k: v for k, v in arguments.items() if k in existing_args}
