@@ -1,5 +1,6 @@
 from ._dataframe import WidgetDataFrame as DataFrame, WidgetSeries as Series, Defaults
 from .io import read_excel, read_csv
+from ._viewer import DataFrameView
 
 __all__ = [
     "DataFrame",
@@ -7,4 +8,5 @@ __all__ = [
     "read_excel",
     "read_csv",
     "Defaults",
+    "DataFrameView",
 ]
