@@ -3,9 +3,11 @@ from typing import Sequence
 import numpy as np
 from numpy.typing import ArrayLike
 from vispy.scene import visuals, ViewBox
+from vispy.color import get_color_dict
 from ._base import LayerItem
 from .._shared_utils import convert_color_code, to_rgba
 
+
 _SYMBOL_MAP = {
     "s": "square",
     "D": "diamond",
@@ -22,8 +24,9 @@ class PlotDataLayer(LayerItem):
 
     @xdata.setter
     def xdata(self, value: Sequence[float]):
-
-        self._visual.set_data
+        x = np.atleast_2d(value)
+        y = self._data[:, 1]
+        self._visual.set_data(np.concatenate([x, y], axis=1))
 
     @property
     def ydata(self) -> np.ndarray:
@@ -31,7 +34,9 @@ class PlotDataLayer(LayerItem):
 
     @ydata.setter
     def ydata(self, value: Sequence[float]):
-        self.native.setData(self.xdata, value)
+        x = self._data[:, 0]
+        y = np.atleast_2d(value)
+        self._visual.set_data(np.concatenate([x, y], axis=1))
 
     @property
     def ndata(self) -> int:
@@ -45,7 +50,7 @@ class PlotDataLayer(LayerItem):
     def name(self, value: str):
         self._name = str(value)
 
-    def add(self, points: np.ndarray | Sequence, **kwargs):
+    def add(self, points: np.ndarray | Sequence):
         """Add new points to the plot data item."""
         points = np.atleast_2d(points)
         if points.shape[1] != 2:
@@ -59,30 +64,32 @@ class PlotDataLayer(LayerItem):
         if isinstance(i, int):
             i = [i]
         sl = list(set(range(self.ndata)) - set(i))
-        xdata = self.xdata[sl]
-        ydata = self.ydata[sl]
-        self.native.setData(xdata, ydata)
+        x = self.xdata[sl]
+        y = self.ydata[sl]
+        self._visual.set_data(np.concatenate([x, y], axis=1))
         return None
 
     @property
     def edge_color(self) -> np.ndarray:
         """Edge color of the data."""
-        return to_rgba(self.native.opts["pen"])
+        col = self._visual._line.color
+        return to_rgba(col)
 
     @edge_color.setter
     def edge_color(self, value: str | Sequence):
         value = convert_color_code(value)
-        self.native.setPen(value, width=self.lw, style=self.ls)
+        self._visual.set_data(edge_color=value)
 
     @property
     def face_color(self) -> np.ndarray:
         """Face color of the data."""
-        return to_rgba(self.native.opts["brush"])
+        col = self._visual._markers._data["face_color"]
+        return to_rgba(col)
 
     @face_color.setter
     def face_color(self, value: str | Sequence):
         value = convert_color_code(value)
-        self.native.setBrush(value)
+        self._visual.set_data(face_color=value)
 
     color = property()
 
@@ -149,3 +156,57 @@ class Scatter(PlotDataLayer):
         )
         self._name = name
         self._visual.update()
+
+
+class Histogram(LayerItem):
+    def __init__(
+        self,
+        viewbox: ViewBox,
+        data: np.ndarray,
+        bins: int = 10,
+        face_color=None,
+        edge_color=None,
+        name: str | None = None,
+    ) -> None:
+        self._viewbox = viewbox
+        self._visual = visuals.Histogram(
+            data,
+            bins=bins,
+            # color=edge_color,
+            parent=self._viewbox.scene,
+        )
+
+        if isinstance(face_color, str):
+            rgb_html = get_color_dict()[face_color][1:]
+            face_color = np.array(
+                [
+                    int(rgb_html[0:2], 16) / 255,
+                    int(rgb_html[2:4], 16) / 255,
+                    int(rgb_html[4:6], 16) / 255,
+                ]
+            )
+
+        if isinstance(edge_color, str):
+            rgb_html = get_color_dict()[edge_color][1:]
+            edge_color = np.array(
+                [
+                    int(rgb_html[0:2], 16) / 255,
+                    int(rgb_html[2:4], 16) / 255,
+                    int(rgb_html[4:6], 16) / 255,
+                ]
+            )
+
+        self._visual.mesh_data.set_face_colors(
+            np.stack([face_color] * self._visual.mesh_data.n_faces, axis=0)
+        )
+        self._visual.mesh_data.set_vertex_colors(
+            np.stack([edge_color] * self._visual.mesh_data.n_vertices, axis=0)
+        )
+        self._visual.mesh_data_changed()
+
+        self._name = name
+        self._visual.update()
+
+    @property
+    def name(self):
+        return self._name
