@@ -4,8 +4,8 @@ import weakref
 import vedo
 import numpy as np
 from .const import Representation
-from ...fields import vfield, HasFields
-from ...types import Color
+from magicclass.fields import vfield, HasFields
+from magicclass.types import Color
 
 _VtkType = TypeVar("_VtkType", bound=vedo.BaseActor)
 
@@ -45,7 +45,7 @@ class VtkComponent(HasFields):
         self._update()
 
     def _update(self):
-        self._parent_ref().window.Render()
+        self._parent_ref().qt_widget.Render()
 
     def __repr__(self) -> str:
         return f"{self.__class__.__name__}<{hex(id(self))}>"
