@@ -9,7 +9,6 @@ from typing import (
     Generic,
 )
 from typing_extensions import Literal, _AnnotatedAlias
-import sys
 from magicgui.widgets import create_widget, Widget
 from magicclass._magicgui_compat import (
     ValueWidget,
@@ -25,12 +24,12 @@ from magicclass.utils import (
     is_instance_method,
     method_as_getter,
     eval_attribute,
+    is_type_like,
 )
 from magicclass.signature import MagicMethodSignature
 from magicclass._gui.mgui_ext import Action, WidgetAction
 
 if TYPE_CHECKING:
-    from magicclass._magicgui_compat import WidgetProtocol
     from typing_extensions import Self
     from magicclass._gui._base import MagicTemplate
     from magicclass._gui.mgui_ext import AbstractAction
@@ -38,11 +37,6 @@ if TYPE_CHECKING:
     _M = TypeVar("_M", bound=MagicTemplate)
     _X = TypeVar("_X", bound=MGUI_SIMPLE_TYPES)
 
-if sys.version_info >= (3, 10):
-    from typing import _BaseGenericAlias
-else:
-    from typing_extensions import _BaseGenericAlias
-
 
 class _FieldObject:
     name: str
@@ -55,6 +49,7 @@ class _FieldObject:
 _W = TypeVar("_W", bound=Widget)
 _V = TypeVar("_V")
 _U = TypeVar("_U")
+_F = TypeVar("_F", bound=Callable)
 
 
 class MagicField(_FieldObject, Generic[_W, _V]):
@@ -233,7 +228,7 @@ class MagicField(_FieldObject, Generic[_W, _V]):
         if is_instance_method(_arg_choices):
             self.options["choices"] = method_as_getter(obj, _arg_choices)
         try:
-            if _is_subclass(constructor, Widget):
+            if _is_magicclass(constructor):
                 widget = constructor(**self.options)
             else:
                 widget = constructor(obj)
@@ -272,7 +267,7 @@ class MagicField(_FieldObject, Generic[_W, _V]):
         Get a widget from ``obj``. This function will be called every time MagicField is referred
         by ``obj.field``.
         """
-        from .._gui import MagicTemplate
+        from magicclass._gui import MagicTemplate
 
         obj_id = id(obj)
         if (widget := self._guis.get(obj_id, None)) is None:
@@ -295,7 +290,7 @@ class MagicField(_FieldObject, Generic[_W, _V]):
         Get an action from ``obj``. This function will be called every time MagicField is referred
         by ``obj.field``.
         """
-        from .._gui import MagicTemplate
+        from magicclass._gui import MagicTemplate
 
         obj_id = id(obj)
         if obj_id in self._guis.keys():
@@ -429,7 +424,7 @@ class MagicField(_FieldObject, Generic[_W, _V]):
         """
         return self.get_action(self.default_object)
 
-    def connect(self, func: Callable) -> Callable:
+    def connect(self, func: _F) -> _F:
         """Set callback function to "ready to connect" state."""
         if not callable(func):
             raise TypeError("Cannot connect non-callable object")
@@ -446,13 +441,14 @@ class MagicField(_FieldObject, Generic[_W, _V]):
         self._callbacks.pop(i)
         return None
 
-    def wraps(
-        self,
-        method: Callable | None = None,
-        *,
-        template: Callable | None = None,
-        copy: bool = False,
-    ):
+    # fmt: off
+    @overload
+    def wraps(self, method: _F, *, template: Callable | None = None, copy: bool = False) -> _F: ...
+    @overload
+    def wraps(self, method: None = ..., *, template: Callable | None = None, copy: bool = False) -> Callable[[_F], _F]: ...
+    # fmt: on
+
+    def wraps(self, method, *, template=None, copy=False):
         """
         Call the ``wraps`` class method of magic class.
 
@@ -487,7 +483,7 @@ class MagicField(_FieldObject, Generic[_W, _V]):
         Callable
             Same method as input, but has updated signature.
         """
-        from .._gui import BaseGui
+        from magicclass._gui import BaseGui
 
         cls = self.constructor
         if not (isinstance(cls, type) and issubclass(cls, BaseGui)):
@@ -648,7 +644,7 @@ def field(
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
+    widget_type: str | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicField[_M, Any]:
@@ -657,7 +653,7 @@ def field(
 
 @overload
 def field(
-    type_of_widget: type[_W],
+    obj: type[_W],
     *,
     name: str | None = None,
     label: str | None = None,
@@ -673,7 +669,7 @@ def field(
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
+    widget_type: str | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicField[ValueWidget, _X]:
@@ -686,7 +682,7 @@ def field(
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
+    widget_type: str | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicField[ValueWidget, _X]:
@@ -695,7 +691,7 @@ def field(
 
 @overload
 def field(
-    obj: Any,
+    obj: Any | None = None,
     *,
     name: str | None = None,
     label: str | None = None,
@@ -712,7 +708,7 @@ def field(
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: str | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicField[Widget, Any]:
@@ -724,7 +720,7 @@ def field(
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
+    widget_type: str | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicField[Widget, Any]:
@@ -786,11 +782,11 @@ def vfield(
 
 @overload
 def vfield(
-    obj: _X,
+    obj: type[_X],
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
+    widget_type: str | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicValueField[ValueWidget, _X]:
@@ -799,11 +795,11 @@ def vfield(
 
 @overload
 def vfield(
-    annotation: type[_X],
+    obj: _X,
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
+    widget_type: str | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicValueField[ValueWidget, _X]:
@@ -829,7 +825,7 @@ def vfield(
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
+    widget_type: str | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicValueField[Widget, Any]:
@@ -841,7 +837,7 @@ def vfield(
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | type[Widget] | None = None,
+    widget_type: str | type[Widget] | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicValueField[Widget, Any]:
@@ -853,7 +849,7 @@ def vfield(
     *,
     name: str | None = None,
     label: str | None = None,
-    widget_type: str | type[WidgetProtocol] | None = None,
+    widget_type: str | None = None,
     options: dict[str, Any] = {},
     record: bool = True,
 ) -> MagicValueField[Widget, Any]:
@@ -934,7 +930,7 @@ def _get_field(
     obj,
     name: str,
     label: str,
-    widget_type: str | type[WidgetProtocol] | None,
+    widget_type: str | None,
     options: dict[str, Any],
     record: bool,
     field_class: type[MagicField],
@@ -945,14 +941,14 @@ def _get_field(
     kwargs = dict(
         name=name, label=label, record=record, annotation=None, options=options
     )
-    if isinstance(obj, (type, _BaseGenericAlias)):
+    if is_type_like(obj):
         if isinstance(obj, _AnnotatedAlias):
-            from ..signature import split_annotated_type
+            from magicclass.signature import split_annotated_type
 
             tp, widget_option = split_annotated_type(obj)
             kwargs.update(annotation=tp)
             options.update(**widget_option)
-        if _is_subclass(obj, Widget):
+        if _is_magicclass(obj):
             if widget_type is not None:
                 raise ValueError("Cannot specify Widget type twice.")
             f = field_class(constructor=obj, widget_type=obj, **kwargs)
@@ -968,8 +964,10 @@ def _get_field(
     return f
 
 
-def _is_subclass(obj: Any, class_or_tuple):
+def _is_magicclass(obj: Any):
+    from magicclass._gui import BaseGui
+
     try:
-        return issubclass(obj, class_or_tuple)
+        return issubclass(obj, (Widget, BaseGui))
     except Exception:
         return False
