@@ -1,17 +1,10 @@
+from logging import *  # noqa
 from .core import (
     getLogger,
-    get_logger_widget,
     debug,
     info,
     warning,
     error,
     fatal,
     critical,
-    DEBUG,
-    INFO,
-    WARNING,
-    ERROR,
-    CRITICAL,
-    FATAL,
-    NOTSET,
 )
