@@ -1,4 +1,5 @@
 from __future__ import annotations
+from contextlib import suppress
 import threading
 import time
 from timeit import default_timer
@@ -19,7 +20,7 @@ from typing import (
 from typing_extensions import TypedDict, ParamSpec
 
 from superqt.utils import create_worker, GeneratorWorker, FunctionWorker
-from qtpy.QtCore import Qt
+from qtpy.QtCore import Qt, QThread, QCoreApplication
 from magicgui.widgets import ProgressBar, Container, Widget, PushButton, Label
 from magicgui.application import use_app
 
@@ -31,7 +32,7 @@ from magicclass.widgets.containers import FrameContainer
 
 if TYPE_CHECKING:
     from magicclass._gui import BaseGui
-    from magicclass._gui.mgui_ext import PushButtonPlus, Action
+    from magicclass._gui.mgui_ext import Clickable
     from magicclass._gui._macro import GuiMacro
     from magicclass.fields import MagicField
     from typing_extensions import Self
@@ -276,10 +277,11 @@ class DefaultProgressBar(FrameContainer, _SupportProgress):
         super().__init__(widgets=[self.progress_label, self.pbar, cnt], labels=False)
 
     def _on_timer_updated(self, _=None):
-        if self._timer.sec < 3600:
-            self.time_label.value = self._timer.format_time("{min:0>2}:{sec:0>2}")
-        else:
-            self.time_label.value = self._timer.format_time()
+        with suppress(RuntimeError):
+            if self._timer.sec < 3600:
+                self.time_label.value = self._timer.format_time("{min:0>2}:{sec:0>2}")
+            else:
+                self.time_label.value = self._timer.format_time()
         return None
 
     def _start_thread(self):
@@ -293,11 +295,12 @@ class DefaultProgressBar(FrameContainer, _SupportProgress):
 
     def _update_timer_label(self):
         """Background thread for updating the progress bar"""
-        while self._running:
-            if self._timer._running:
-                self._time_signal.emit()
+        with suppress(RuntimeError):
+            while self._running:
+                if self._timer._running:
+                    self._time_signal.emit()
 
-            time.sleep(0.1)
+                time.sleep(0.1)
         return None
 
     def _finish(self):
@@ -329,27 +332,37 @@ class DefaultProgressBar(FrameContainer, _SupportProgress):
 
     def show(self):
         parent = self.native.parent()
-        self._CONTAINER.append(self)
-        if self._CONTAINER.parent is not parent:
-            self._CONTAINER.native.setParent(
-                parent,
-                self._CONTAINER.native.windowFlags(),
-            )
-        if not self._CONTAINER.visible:
-            self._CONTAINER.show()
-            move_to_screen_center(self._CONTAINER.native)
+        with suppress(RuntimeError):
+            if _is_main_thread():
+                self._CONTAINER.append(self)
+                if self._CONTAINER.parent is not parent:
+                    self._CONTAINER.native.setParent(
+                        parent,
+                        self._CONTAINER.native.windowFlags(),
+                    )
+            if not self._CONTAINER.visible:
+                self._CONTAINER.show()
+                move_to_screen_center(self._CONTAINER.native)
         return None
 
     def close(self):
+        i = -1
         for i, wdt in enumerate(self._CONTAINER):
             if wdt is self:
                 break
-        self._CONTAINER.pop(i)
-        self._CONTAINER.height = 1  # minimize height
-        if len(self._CONTAINER) == 0:
-            self._CONTAINER.close()
+        if i < 0:
+            return None
+        with suppress(RuntimeError):
+            if _is_main_thread():
+                self._CONTAINER.pop(i)
+                self._CONTAINER.height = 1  # minimize height
+                if len(self._CONTAINER) == 0:
+                    self._CONTAINER.close()
         return None
 
+    def hide_footer(self):
+        self.footer[1].visible = self.footer[2].visible = False
+
     def set_description(self, desc: str):
         """Set description as the label of the progressbar."""
         self.progress_label.value = desc
@@ -367,8 +380,7 @@ class DefaultProgressBar(FrameContainer, _SupportProgress):
         self._worker.started.connect(self._start_thread)
         if not isinstance(self._worker, GeneratorWorker):
             # FunctionWorker does not have yielded/aborted signals.
-            self.footer[1].visible = False
-            self.footer[2].visible = False
+            self.hide_footer()
             return None
         # initialize abort_button
         self.abort_button.text = "Abort"
@@ -423,7 +435,7 @@ class Aborted(RuntimeError):
 ProgressBarLike = Union[ProgressBar, _SupportProgress]
 
 
-class thread_worker:
+class thread_worker(Generic[_P]):
     """Create a worker in a superqt/napari style."""
 
     _DEFAULT_PROGRESS_BAR = DefaultProgressBar
@@ -439,7 +451,7 @@ class thread_worker:
         f: Callable[_P, _R1] | None = None,
         *,
         ignore_errors: bool = False,
-        progress: ProgressDict | None = None,
+        progress: ProgressDict | bool | None = None,
     ) -> None:
         self._func: Callable[_P, _R1] | None = None
         self._callback_dict_ = {
@@ -464,7 +476,7 @@ class thread_worker:
                 progress = {}
 
             progress.setdefault("desc", None)
-            progress.setdefault("total", 0)
+            progress.setdefault("total", self.__class__._DEFAULT_TOTAL)
             progress.setdefault("pbar", None)
 
         self._progress = progress
@@ -477,8 +489,9 @@ class thread_worker:
     def with_progress(
         cls,
         desc: str | Callable | None = None,
-        total: str | Callable | int = 0,
+        total: str | Callable | int | None = None,
         pbar: ProgressBar | _SupportProgress | MagicField | None = None,
+        **kwargs,
     ) -> Self:
         """
         Configure the progressbar.
@@ -496,14 +509,15 @@ class thread_worker:
             Progressbar object.
         """
         self = cls()
-
-        progress = dict(desc=desc, total=total, pbar=pbar)
+        if total is None:
+            total = cls._DEFAULT_TOTAL
+        progress = dict(desc=desc, total=total, pbar=pbar, **kwargs)
 
         self._progress = progress
         return self
 
     @classmethod
-    def set_default(cls, pbar_cls: Callable | str) -> Self:
+    def set_default(cls, pbar_cls: Callable | str):
         """
         Set the default progressbar class.
 
@@ -566,7 +580,7 @@ class thread_worker:
         return None
 
     @overload
-    def __call__(self, f: Callable[_P, _R1]) -> thread_worker:
+    def __call__(self, f: Callable[_P, _R1]) -> thread_worker[_P]:
         ...
 
     @overload
@@ -582,7 +596,7 @@ class thread_worker:
         else:
             return self._func(*args, **kwargs)
 
-    def __get__(self, gui: BaseGui, objtype=None):
+    def __get__(self, gui: BaseGui, objtype=None) -> Callable[_P, Any]:
         if gui is None:
             return self
 
@@ -596,28 +610,53 @@ class thread_worker:
         self._objects[gui_id] = _create_worker  # cache
         return _create_worker
 
+    def button(self, gui: BaseGui) -> Clickable:
+        """The corresponding button object."""
+        return gui[self._func.__name__]
+
     def _create_qt_worker(
-        self, gui, *args, **kwargs
+        self, gui: BaseGui, *args, **kwargs
     ) -> FunctionWorker | GeneratorWorker:
         """Create a worker object."""
+        if self.is_generator:
+
+            def _run(*args, **kwargs):
+                with gui.macro.blocked():
+                    out = yield from self._func.__get__(gui)(*args, **kwargs)
+                return out
+
+        else:
+
+            def _run(*args, **kwargs):
+                with gui.macro.blocked():
+                    out = self._func.__get__(gui)(*args, **kwargs)
+                return out
+
         worker = create_worker(
-            self._func.__get__(gui),
+            _run,
             _ignore_errors=self._ignore_errors,
             _start_thread=False,
             *args,
             **kwargs,
         )
+
+        @worker.returned.connect
+        def _on_returned(out):
+            if gui.macro.active and self._recorder is not None:
+                self._recorder(gui, out, *args, **kwargs)
+            return
+
         return worker
 
-    def _create_method(self, gui: BaseGui):
-        from ..fields import MagicField
+    def _create_method(self, gui: BaseGui) -> Callable[_P, None]:
+        from magicclass.fields import MagicField
 
         @wraps(self)
         def _create_worker(*args, **kwargs):
             # create a worker object
             worker = self._create_qt_worker(gui, *args, **kwargs)
             is_generator = isinstance(worker, GeneratorWorker)
-
+            pbar = None
             if self._progress:
                 # prepare progress bar
                 _pbar = self._progress["pbar"]
@@ -642,10 +681,6 @@ class thread_worker:
 
             self._bind_callbacks(worker, gui)
 
-            # bind macro-recorder if exists
-            if self._recorder is not None:
-                worker.returned.connect(lambda _: self._recorder(gui, *args, **kwargs))
-
             if self._progress:
                 if not getattr(pbar, "visible", False):
                     # return the progressbar to the initial state
@@ -660,31 +695,55 @@ class thread_worker:
                 if hasattr(pbar, "set_title"):
                     pbar.set_title(self._func.__name__.replace("_", " "))
 
-            _obj: PushButtonPlus | Action = gui[self._func.__name__]
-            if _obj.running:
-                worker.errored.connect(
-                    partial(gui._error_mode.get_handler(), parent=gui)
-                )
+            if self.button(gui).running:
+
+                @worker.errored.connect
+                def _on_error(err: Exception):
+                    # NOTE: Exceptions are raised in other thread so context manager
+                    # cannot catch them. Macro has to be reactived here.
+                    gui._error_mode.get_handler()(err, parent=gui)
+                    gui.macro.active = True
+
                 if is_generator:
-                    worker.aborted.connect(
-                        gui._error_mode.wrap_handler(Aborted.raise_, parent=gui)
-                    )
+
+                    @worker.aborted.connect
+                    def _on_abort():
+                        gui._error_mode.wrap_handler(Aborted.raise_, parent=gui)()
+                        gui.macro.active = True
 
                 worker.start()
             else:
                 # If function is called from script, some events must get processed by
                 # the application while keep script stopping at each line of code.
-                app = use_app()
-                worker.returned.connect(app.process_events)
-                worker.started.connect(app.process_events)
-                if isinstance(worker, GeneratorWorker):
-                    worker.yielded.connect(app.process_events)
-                worker.run()
+                self._run_blocked(gui, worker, pbar)
 
             return None
 
+        _create_worker.__self__ = gui
         return _create_worker
 
+    def _run_blocked(
+        self,
+        gui: BaseGui,
+        worker: FunctionWorker | GeneratorWorker,
+        pbar: ProgressBar | None,
+    ):
+        app = use_app()
+        worker.started.connect(app.process_events)
+        worker.finished.connect(app.process_events)
+        if isinstance(worker, GeneratorWorker):
+            worker.yielded.connect(app.process_events)
+        try:
+            worker.run()
+
+        except KeyboardInterrupt:
+            if isinstance(pbar, DefaultProgressBar):
+                pbar._abort_worker()
+            else:
+                worker.quit()
+            worker.finished.emit()
+            gui._error_mode.wrap_handler(Aborted.raise_, parent=gui)()
+
     def _get_method_signature(self) -> inspect.Signature:
         sig = self.__signature__
         params = list(sig.parameters.values())[1:]
@@ -725,7 +784,7 @@ class thread_worker:
 
     def _find_progressbar(self, gui: BaseGui, desc: str | None = None, total: int = 0):
         """Find available progressbar. Create a new one if not found."""
-        from ..fields import MagicField
+        from magicclass.fields import MagicField
 
         gui_id = id(gui)
         if gui_id in self._progressbars:
@@ -746,7 +805,7 @@ class thread_worker:
 
         if _pbar is None:
             _pbar = self.__class__._DEFAULT_PROGRESS_BAR(max=total)
-            if isinstance(_pbar, Widget) and _pbar.parent is None:
+            if isinstance(_pbar, Widget) and _pbar.parent is None and _is_main_thread():
                 # Popup progressbar as a splashscreen if it is not a child widget.
                 _pbar.native.setParent(gui.native, self.__class__._WINDOW_FLAG)
 
@@ -764,25 +823,28 @@ class thread_worker:
         _desc = self._progress["desc"]
         _total = self._progress["total"]
 
-        all_args = None
+        all_args = None  # all the argument of the function
         # progress bar description
         if callable(_desc):
             arguments = self.__signature__.bind(gui, *args, **kwargs)
             arguments.apply_defaults()
             all_args = arguments.arguments
-            desc = _desc(**all_args)
+            desc = _desc(**_filter_args(_desc, all_args))
         else:
             desc = str(_desc or self._func.__name__)
 
         # total number of steps
-        if isinstance(_total, str):
+        if isinstance(_total, str) or callable(_total):
             if all_args is None:
                 arguments = self.__signature__.bind(gui, *args, **kwargs)
                 arguments.apply_defaults()
                 all_args = arguments.arguments
-            total = eval(_total, {}, all_args)
-        elif callable(_total):
-            total = _total(gui)
+            if isinstance(_total, str):
+                total = eval(_total, {}, all_args)
+            elif callable(_total):
+                total = _total(**_filter_args(_total, all_args))
+            else:
+                raise RuntimeError("Unreachable.")
         elif isinstance(_total, int):
             total = _total
         else:
@@ -850,13 +912,31 @@ def close_pbar(pbar: ProgressBarLike):
 
 def increment(pbar: ProgressBarLike):
     """Increment progressbar."""
-    if pbar.value == pbar.max:
-        pbar.max = 0
-    else:
-        pbar.value += 1
+    with suppress(RuntimeError):
+        if pbar.value == pbar.max:
+            pbar.max = 0
+        else:
+            pbar.value += 1
     return None
 
 
+def _filter_args(fn: Callable, arguments: dict[str, Any]) -> dict[str, Any]:
+    sig = inspect.signature(fn)
+    params = sig.parameters
+    nparams = len(params)
+    if nparams == 0:
+        return {}
+    if list(sig.parameters.values())[-1] == inspect.Parameter.VAR_KEYWORD:
+        return arguments
+    existing_args = set(sig.parameters.keys())
+    return {k: v for k, v in arguments.items() if k in existing_args}
+
+
+def _is_main_thread() -> bool:
+    """True if the current thread is the main thread."""
+    return QThread.currentThread() is QCoreApplication.instance().thread()
+
+
 class Callback:
     """Callback object that can be recognized by thread_worker."""
 
@@ -873,7 +953,7 @@ class Callback:
 
     def __call__(self, *args, **kwargs) -> Callback:
         """Return a partial callback."""
-        return self.__class__(partial(self._func, *args, *kwargs))
+        return self.__class__(partial(self._func, *args, **kwargs))
 
     def __get__(self, obj, type=None) -> Callback:
         if obj is None:
@@ -881,4 +961,4 @@ class Callback:
         return self(obj)
 
 
-to_callback = thread_worker.to_callback
+to_callback = thread_worker.to_callback  # function version
