@@ -33,9 +33,13 @@ from .plot import Figure, SeabornFigure
 from .separator import Separator
 from .utils import FreeWidget
 from .logger import Logger
+from .codeedit import CodeEdit
+from .toggle_switch import ToggleSwitch
+from .eval import EvalLineEdit
 
 __all__ = [
     "ButtonContainer",
+    "CodeEdit",
     "ColorEdit",
     "ColorSlider",
     "ConsoleTextEdit",
@@ -43,6 +47,7 @@ __all__ = [
     "CollapsibleContainer",
     "DictWidget",
     "DraggableContainer",
+    "EvalLineEdit",
     "FrameContainer",
     "Figure",
     "FloatRangeSlider",
@@ -64,5 +69,6 @@ __all__ = [
     "SpreadSheet",
     "StackedContainer",
     "TabbedContainer",
+    "ToggleSwitch",
     "ToolBoxContainer",
 ]
