@@ -150,10 +150,36 @@ def set_design(
     return wrapper
 
 
-def do_not_record(method: F) -> F:
-    """Wrapped method will not be recorded in macro."""
-    upgrade_signature(method, additional_options={"record": False})
-    return method
+@overload
+def do_not_record(method: None = None, recursive: bool = True) -> Callable[[F], F]:
+    ...
+
+
+@overload
+def do_not_record(method: F, recursive: bool = True) -> F:
+    ...
+
+
+def do_not_record(method=None, recursive=False):
+    """
+    Wrapped method will not be recorded in macro.
+
+    Parameters
+    ----------
+    recursive : bool, default is False
+        If True, all recordable methods called inside this method will also be
+        suppressed.
+    """
+
+    def wrapper(f):
+        if recursive:
+            record = "all-false"
+        else:
+            record = "false"
+        upgrade_signature(f, additional_options={"record": record})
+        return f
+
+    return wrapper if method is None else wrapper(method)
 
 
 def bind_key(*key) -> Callable[[F], F]:
@@ -289,14 +315,17 @@ def nogui(method: F) -> F:
 
 
 if TYPE_CHECKING:
-    from typing import Protocol
+    from typing_extensions import ParamSpec
 
-    class PreviewFunction(Protocol):
+    _P = ParamSpec("_P")
+    _R = TypeVar("_R")
+
+    class PreviewFunction(Callable[_P, _R]):
         __name__: str
         __qualname__: str
 
-        def __call__(self, *args, **kwargs):
-            ...
+        def __call__(self, *args: _P.args, **kwargs: _P.kwargs) -> _R:
+            """Run function."""
 
         def during_preview(self, f: F) -> F:
             """Wrapped function will be used as a context manager during preview."""
@@ -391,6 +420,16 @@ def impl_preview(
             return _func
 
         def _wrapper(preview: F) -> F:
+            _is_generator = inspect.isgeneratorfunction(preview)
+            if _is_generator:
+                _preview_context_generator = preview
+                preview = lambda: None
+                for attr in ["__name__", "__qualname__", "__module__"]:
+                    if hasattr(_preview_context_generator, attr):
+                        setattr(
+                            preview, attr, getattr(_preview_context_generator, attr)
+                        )
+
             _filter = _get_arg_filter(preview)
             _preview = _impl_arg_filter(preview, _filter)
             _preview.__wrapped__ = preview
@@ -414,6 +453,9 @@ def impl_preview(
                 append_preview(target_func, _preview, text=text, auto_call=auto_call)
 
             # add method
+            if _is_generator:
+                _set_during_preview(_preview_context_generator)
+                return _preview_context_generator
             preview.during_preview = _set_during_preview
             return preview
 
@@ -430,6 +472,48 @@ def impl_preview(
 
 mark_preview = impl_preview
 
+
+def setup_function_gui(target: Callable):
+    """
+    Mark a function as a setup function for a FunctionGui.
+
+    Function decorated with ``setup_function_gui(func)`` will be called when the
+    FunctionGui widget for method ``func`` was built. This decorator is used to
+    define complicated setting of a FunctionGui, which is not achievable by
+    simple configurations.
+
+    >>> @magicclass
+    >>> class A:
+    ...     def func(self, x: int, xmax: int):
+    ...         # target function
+    ...
+    ...     @setup_function_gui(func)
+    ...     def _setup_func(self, gui: FunctionGui):
+    ...         @gui.xmax.changed.connect
+    ...         def _(xmax_value):
+    ...             gui.x.max = xmax_value
+    """
+
+    def wrapper(setup: Callable[[BaseGui, FunctionGui], None]):
+        setup_qualname = setup.__qualname__
+        target_qualname = target.__qualname__
+        if setup_qualname.split(".")[-2] != target_qualname.split(".")[-2]:
+            # have to search for the proper parent instance
+            def _setup(self: BaseGui, mgui: FunctionGui):
+                prev_ns = setup_qualname.split(".")[-2]
+                while self.__class__.__name__ != prev_ns:
+                    self = self.__magicclass_parent__
+                return setup(self, mgui)
+
+        else:
+            _setup = setup
+        upgrade_signature(target, additional_options={"setup": _setup})
+        upgrade_signature(setup, additional_options={"nogui": True})
+        return setup
+
+    return wrapper
+
+
 _Fn = TypeVar("_Fn", bound=Callable[[FunctionGui], Any])
 
 
