@@ -1,4 +1,4 @@
-from magicclass import magicclass, magicmenu
+from magicclass import magicclass, magicmenu, get_button
 
 # PushButtonPlus.running should be True only if it is called from GUI.
 
@@ -9,12 +9,12 @@ def test_running():
             self.last_running = None
 
         def func(self):
-            self.last_running = self["func"].running
+            self.last_running = get_button(self.func).running
 
     ui = A()
     ui.func()
     assert ui.last_running == False
-    ui["func"].changed()
+    get_button(ui.func).changed()
     assert ui.last_running == True
 
 
@@ -30,10 +30,10 @@ def test_wrapped_running():
 
         @B.wraps
         def func(self):
-            self.last_running = self["func"].running
+            self.last_running = get_button(self.func).running
 
     ui = A()
     ui.func()
     assert ui.last_running == False
-    ui.B["func"].changed()
+    get_button(ui.func).changed()
     assert ui.last_running == True
